﻿using System;
using System.Windows.Forms;
using System.Data;
using Cybele.Thinfinity;

namespace fishfact4
{
    public partial class FormMain : Form
    {
        private IJSObject mRemoteDB = null;
        private IJSObject mRemoteLayout = null;

        public FormMain() {
            InitializeComponent();
            // Creates "layout" JSObject
            mRemoteLayout = new JSObject("layout");

            // Adds a "bingSearch" property
            mRemoteLayout.Properties.Add("bingSearch")
                .OnGet(new JSBinding(          // Adds a "getter" to the returned IJSProperty
                // This anonymous procedure do the actual get
                delegate(IJSObject Parent, IJSProperty Prop) {
                    Prop.AsBool = chkBingSearch.Checked;
                }));

            // Adds a "windowVisible" property
            mRemoteLayout.Properties.Add("windowVisible")
                .OnGet(new JSBinding(         // Adds a "getter" to the returned IJSProperty
                // This anonymous procedure do the actual get
                delegate(IJSObject Parent, IJSProperty Prop) {
                    Prop.AsBool = Visible;
                }))
                .OnSet(new JSBinding(        // Adds a "setter" to the returned IJSProperty
                // This anonymous procedure do the actual set
                delegate(IJSObject Parent, IJSProperty Prop) {
                    Visible = Boolean.Parse(Prop.AsString);
                    if (!Visible)
                        chkBingSearch.Checked = true;
                }));

            // Applies the object model and sends it to the web browser
            mRemoteLayout.ApplyModel();
        }

        public IJSObject RemoteDB {
            get {
                if (mRemoteDB == null) {
                    // Creates the "db" JSObject
                    mRemoteDB = new JSObject("db");

                    var allowedTypes = new[] { typeof(String), typeof(Byte), typeof(Char), typeof(Decimal), typeof(Double) };

                    // Iterates over the dataset fields
                    foreach (DataColumn column in dbdemosDataSet.BIOLIFE.Columns) {
                        // Bypasses some data types
                        if (Array.IndexOf(allowedTypes, column.DataType) >= 0) {
                            // Creates the property with the name of the field.
                            mRemoteDB.Properties.Add(column.ColumnName)     // Returns a IJSProperty
                                .OnGet(new JSBinding(                       // Adds a "getter" to the returned IJSProperty
                                    // This anonymous procedure do the actual get
                                    delegate(IJSObject AObj, IJSProperty AProp) {
                                        DataColumn col = dbdemosDataSet.BIOLIFE.Columns[AProp.Name];
                                        string value = dbdemosDataSet.BIOLIFE.Rows[bIOLIFEBindingSource.Position][col].ToString();

                                        if ((col.DataType == typeof(String)) || (col.DataType == typeof(Byte)) || (col.DataType == typeof(Char))) {
                                            AProp.AsString = value;
                                        }
                                        else {
                                            AProp.AsFloat = float.Parse(value);
                                        }
                                    }
                                ));
                        }
                    }

                    // Adds a method to command the dataset navigation from Javascript
                    mRemoteDB.Methods.Add("navigate")                   // Returns a IJSMethod
                      .AddArgument("target", IJSDataType.JSDT_STRING)   // Adds a string parameter.
                      .OnCall(new JSCallback(                           // Adds the callback
                        // this anonymous procedure to do the actual execution
                        delegate(IJSObject Parent, IJSMethod Method) {
                            bool RV = true;
                            string Target = Method.Arguments["target"].AsString;

                            if (Target == "first")
                                bIOLIFEBindingSource.MoveFirst();
                            else if (Target == "last")
                                bIOLIFEBindingSource.MoveLast();
                            else if (Target == "prev")
                                bIOLIFEBindingSource.MovePrevious();
                            else if (Target == "next")
                                bIOLIFEBindingSource.MoveNext();
                            else
                                RV = false;
                            Method.ReturnValue.AsBool = RV;
                        }))
                      .ReturnValue.DataType = IJSDataType.JSDT_BOOL;   // Sets the return type

                    // Applies the object model and sends it to the web browser
                    mRemoteDB.ApplyModel();
                }
                return mRemoteDB;
            }
        }

        private void FormMain_Load(object sender, EventArgs e) {
            // TODO: This line of code loads data into the 'dbdemosDataSet.BIOLIFE' table. You can move, or remove it, as needed.
            this.bIOLIFETableAdapter.Fill(this.dbdemosDataSet.BIOLIFE);

        }

        private void bIOLIFEBindingSource_PositionChanged(object sender, EventArgs e) {
            // Calls all mRemoteDB's properties "getters"
            RemoteDB.ApplyChanges();
        }

        private void chkBingSearch_CheckedChanged(object sender, EventArgs e) {
            // Calls all mRemoteLayout's properties "getters"
            if (mRemoteLayout != null) {
                mRemoteLayout.ApplyChanges();
            }
        }
    }
}
