
//---------------------------------------------------------------------------

// This software is Copyright (c) 2011 Embarcadero Technologies, Inc.
// You may only use this software if you are an authorized licensee
// of Delphi, C++Builder or RAD Studio (Embarcadero Products).
// This software is considered a Redistributable as defined under
// the software license agreement that comes with the Embarcadero Products
// and is subject to that software license agreement.

//---------------------------------------------------------------------------
unit Ffactwin;

{ This application shows how to display Paradox style memo and graphic
 fields in a form. Table1's DatabaseName property should point to the
 Delphi sample database. Table1's TableName property should be set to
 the BIOLIFE table. }

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, DBCtrls, DBGrids, DB, Buttons, Grids, ExtCtrls,
  {$IFDEF VER280}
  IBX.IBDatabase, IBX.IBCustomDataSet,IBX.IBTable,
  {$ELSE}
  DBTables,IBDatabase, IBCustomDataSet, IBTable,
  {$ENDIF}
  Vcl.Styles, Vcl.Themes, Vcl.ComCtrls,
  VirtualUI_SDK;

type
  TForm1 = class(TForm)
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    IBTable1: TIBTable;
    IBDatabase1: TIBDatabase;
    IBTransaction1: TIBTransaction;
    IBTable1CATEGORY: TIBStringField;
    IBTable1COMMON_NAME: TIBStringField;
    IBTable1SPECIES_NAME: TIBStringField;
    IBTable1LENGTH__CM_: TFloatField;
    IBTable1LENGTH_IN: TFloatField;
    IBTable1NOTES: TMemoField;
    IBTable1GRAPHIC: TBlobField;
    Splitter2: TSplitter;
    pnlDetail: TPanel;
    Panel3: TPanel;
    DBMemo1: TDBMemo;
    Panel1: TPanel;
    DBLabel1: TDBText;
    DBImage1: TDBImage;
    Panel2: TPanel;
    DBCommonName: TDBText;
    DBSpeciesName: TDBText;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    // Declares the JSObject
    FRemoteDB : IJSObject;
    function GetRemoteDB: IJSObject;
  public
    { Public declarations }
    property RemoteDB:IJSObject read GetRemoteDB;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  IBDatabase1.DatabaseName :=  ExpandFileName('..\\..\\data\\dbdemos.gdb');
  IBDatabase1.Connected := true;
  IBTransaction1.Active := true;
  IBTable1.Active := true;
end;

function TForm1.GetRemoteDB:IJSObject;
var
  n : Integer;
  Field : TField;
begin
  if not Assigned(FRemoteDB) then begin
    // Creates the "db" JSObject
    FRemoteDB := TJSObject.Create('db');

    // Iterates over the dataset fields
    for n := 0 to DataSource1.DataSet.Fields.Count-1 do begin
      Field := DataSource1.DataSet.Fields[n];

      // Bypasses some data types
      if Field.DataType in [ftString, ftFixedChar, ftMemo,ftWideString,ftFloat,ftCurrency] then

        // Creates the property with the name of the field.
        FRemoteDB.Properties.Add(Field.FullName) // Returns a IJSProperty
        .OnGet(TJSBinding.Create(                // Adds a "getter" to the returned IJSProperty
            // This anonymous procedure do the actual get
            procedure(const AObj:IJSObject;const AProp:IJSProperty)
            begin
              Field := DataSource1.DataSet.FieldByName(AProp.Name);

              case Field.DataType of
                ftString, ftFixedChar, ftMemo,ftWideString:
                  AProp.AsString := Field.AsString;
                ftFloat,ftCurrency:
                  AProp.AsFloat := Field.AsFloat;
              end;
            end));
    end;

    // Applies the object model and sends it to the web browser
    FRemoteDB.ApplyModel;
  end;
  Result := FRemoteDB;
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  // Calls all RemoteDB's properties "getters"
  RemoteDB.ApplyChanges;
end;

end.
