﻿Imports System.Windows.Forms
Imports System.IO
Imports fishfact5.Cybele.Thinfinity

Public Class FormMain
    Private mVirualUI As VirtualUI
    Private mRemoteDB As IJSObject = Nothing
    Private mRemoteLayout As IJSObject = Nothing

    Public Sub New()
        InitializeComponent()
        AppDomain.CurrentDomain.SetData("DataDirectory", Path.GetFullPath("..\\..\\..\\..\\data"))

        mVirualUI = New VirtualUI()
        mVirualUI.Start()

        ' Creates "layout" JSObject
        mRemoteLayout = New JSObject("layout")

        ' Adds a "bingSearch" property
        ' Adds a "getter" to the returned IJSProperty (.OnGet)
        ' The anonymous Sub do the actual get
        mRemoteLayout.Properties.Add("bingSearch") _
            .OnGet(New JSBinding( _
                       Sub(Parent As IJSObject, Prop As IJSProperty)
                           Prop.AsBool = chkBingSearch.Checked
                       End Sub))

        ' Adds a "windowVisible" property
        ' Adds a "getter" to the returned IJSProperty (.OnGet)
        ' The anonymous sub do the actual get
        ' Adds a "setter" to the returned IJSProperty (.OnSet)
        ' The anonymous Sub do the actual set
        mRemoteLayout.Properties.Add("windowVisible") _
            .OnGet(New JSBinding( _
                   Sub(Parent As IJSObject, Prop As IJSProperty)
                       Prop.AsBool = Visible
                   End Sub)) _
            .OnSet(New JSBinding( _
                   Sub(Parent As IJSObject, Prop As IJSProperty)
                       Visible = Boolean.Parse(Prop.AsString)
                       If Not Visible Then
                           rbDesktopMode.Checked = True
                           chkBingSearch.Checked = True
                       End If
                   End Sub))

        ' Properties needed to embed the Bing Search iframe
        ' Adds a "windowId" property with the calculated div id containing this window
        mRemoteLayout.Properties.Add("windowId").AsString = String.Format("virtualui_canvas_{0}", Handle)

        ' Adds a "bounds" property with the coordinates where to embed the iframe
        ' Adds a "getter" to the returned IJSProperty (.OnGet)
        ' The anonymous Sub do the actual get
        mRemoteLayout.Properties.Add("bounds") _
            .OnGet(New JSBinding( _
                   Sub(AObj As IJSObject, AProp As IJSProperty)
                       Dim p As Point = pnlDetail.PointToScreen(New Point(0, 0))
                       p.Offset(-Left, -Top)
                       ' Returns a JSON object
                       AProp.AsJSON = "{" & String.Format("""left"":{0},""top"":{1},""width"":{2},""height"":{3}", p.X, p.Y, pnlDetail.Width, pnlDetail.Height) & "}"
                   End Sub))

        ' Adds the "embed" event with 1 boolean argument
        mRemoteLayout.Events.Add("embed").AddArgument("value", IJSDataType.JSDT_BOOL)

        ' Applies the object model and s}s it to the web browser
        mRemoteLayout.ApplyModel()
    End Sub

    Public ReadOnly Property RemoteDB() As IJSObject
        Get
            If mRemoteDB Is Nothing Then
                ' Creates the "db" JSObject
                mRemoteDB = New JSObject("db")

                Dim allowedTypes = New Type() {GetType(String), GetType(Byte), GetType(Char), GetType(Decimal), GetType(Double)}

                ' Iterates over the dataset fields
                For Each column As DataColumn In dbdemosDataSet.BIOLIFE.Columns
                    ' Bypasses some data types
                    If Array.IndexOf(allowedTypes, column.DataType) >= 0 Then
                        ' Creates the property with the name of the field.
                        ' Returns a IJSProperty
                        ' Adds a "getter" to the returned IJSProperty (.OnGet)
                        ' The anonymous Sub do the actual get
                        mRemoteDB.Properties.Add(column.ColumnName) _
                            .OnGet(New JSBinding( _
                                   Sub(AObj As IJSObject, AProp As IJSProperty)
                                       Dim col As DataColumn = DbdemosDataSet.BIOLIFE.Columns(AProp.Name)
                                       Dim value As String = DbdemosDataSet.BIOLIFE.Rows(BIOLIFEBindingSource.Position)(col).ToString()

                                       If (col.DataType = GetType(String)) OrElse (col.DataType = GetType(Byte)) OrElse (col.DataType = GetType(Char)) Then
                                           AProp.AsString = value
                                       Else
                                           AProp.AsFloat = Single.Parse(value)
                                       End If
                                   End Sub))
                    End If
                Next

                ' Adds a method to command the dataset navigation from Javascript ("navigate")
                ' Returns a IJSMethod
                ' Adds a string parameter ("target")
                ' Adds the callback (.OnCall)
                ' The anonymous Sub to do the actual execution
                mRemoteDB.Methods.Add("navigate") _
                    .AddArgument("target", IJSDataType.JSDT_STRING) _
                    .OnCall(New JSCallback( _
                            Sub(Parent As IJSObject, Method As IJSMethod)
                                Dim RV As Boolean = True
                                Dim Target As String = Method.Arguments("target").AsString
                                If Target = "first" Then
                                    BIOLIFEBindingSource.MoveFirst()
                                ElseIf Target = "last" Then
                                    BIOLIFEBindingSource.MoveLast()
                                ElseIf Target = "prev" Then
                                    BIOLIFEBindingSource.MovePrevious()
                                ElseIf Target = "next" Then
                                    BIOLIFEBindingSource.MoveNext()
                                Else
                                    RV = False
                                End If
                                Method.ReturnValue.AsBool = RV
                            End Sub)) _
                    .ReturnValue.DataType = IJSDataType.JSDT_BOOL  ' Sets the return type

                ' Applies the object model and s}s it to the web browser
                mRemoteDB.ApplyModel()
            End If
            Return mRemoteDB
        End Get
    End Property

    Private Sub FormMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Me.BIOLIFETableAdapter.Fill(Me.DbdemosDataSet.BIOLIFE)
    End Sub

    Private Sub BIOLIFEBindingSource_PositionChanged(sender As Object, e As EventArgs) Handles BIOLIFEBindingSource.PositionChanged
        ' Calls all RemoteDB's properties "getters"
        RemoteDB.ApplyChanges()
    End Sub

    Private Sub chkBingSearch_CheckedChanged(sender As Object, e As EventArgs) Handles chkBingSearch.CheckedChanged
        rbEmbedMode.Enabled = chkBingSearch.Checked
        rbDesktopMode.Enabled = chkBingSearch.Checked
        LayoutChanged(sender, e)
    End Sub

    Private Sub rbDesktopMode_CheckedChanged(sender As Object, e As EventArgs) Handles rbDesktopMode.CheckedChanged
        If (mVirualUI IsNot Nothing) Then
            If (mVirualUI.Active And rbDesktopMode.Checked) Then
                ' Fires the 'embed' event
                mRemoteLayout.Events("embed").ArgumentAsBool("value", rbEmbedMode.Checked).Fire()
                LayoutChanged(sender, e)
            End If
        End If
    End Sub

    Private Sub rbEmbedMode_CheckedChanged(sender As Object, e As EventArgs) Handles rbEmbedMode.CheckedChanged
        If (mVirualUI IsNot Nothing) Then
            If (mVirualUI.Active And rbEmbedMode.Checked) Then
                ' Fires the 'embed' event
                mRemoteLayout.Events("embed").ArgumentAsBool("value", rbEmbedMode.Checked).Fire()
                LayoutChanged(sender, e)
            End If
        End If
    End Sub

    Private Sub LayoutChanged(sender As Object, e As EventArgs) Handles MyBase.Shown, pnlDetail.Resize
        ' Calls all RemoteLayout's properties "getters"
        If (mRemoteLayout IsNot Nothing) Then
            mRemoteLayout.ApplyChanges()
        End If
    End Sub

End Class
